package org.eso.dfs.services.data;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eso.dfs.services.ServiceException;

/**
 * @author Nick Kornweibel
 *
 * This is the singleton object for the AssociationService. The
 * service maintains a vector of handlers that form a "modified"
 * chain of responsibility pattern. In this model the Service (this singleton)
 * calls the successor of each handler, rather than the handler performing the
 * call to the successor. In addition, all handlers are called to process the
 * request, rather than stopping at the first handler that can process the
 * request.
 */
public class AssociationService {

    /**
     * Set of handlers in the chain.
     */
    private List handlerChain = new ArrayList();
    
    /**
     * Singleton instance of association service.
     */
    private static AssociationService instance;
    
    /**
     * Constructor for AssociationService.
     */
    private AssociationService() {
        super();
    }
    
    /**
     * Get the singleton instance. It is synchronized to ensure
     * creation of the singleton is atomic.
     * @return AssociationService singleton instance.
     */
    public static synchronized AssociationService getInstance() {
        if (instance == null) {
            instance = new AssociationService();
        }
        return instance;
    }
    
    /**
     * Determine assocation data for the node array.
     * If there is in error an exception is thrown. All handlers will
     * be called to process the array and the resulting exception will
     * be the "summary" of all errors reported.
     * @param nodex Array of AssociationNodes.
     * @throws ServiceException if errors are reported by handlers. The
     * stack trace information for this exception does not identify source
     * of the error, it is more a means of communicating errors.
     */
    public void processNodes(AssociationNode[] nodes) throws ServiceException {
        Iterator i = handlerChain.iterator();
        StringBuffer errorReport = new StringBuffer();
        while (i.hasNext()) {
            AssociationHandler h = (AssociationHandler)i.next();
            try {
                h.processNodes(nodes);
            } catch (ServiceException e) {
                errorReport.append(e.toString()+"\n");
            }
        }
        if (errorReport.length() > 0) {
            throw new ServiceException(errorReport.toString());
        }
    }
    
    
    /**
     * Add the listed files to the association model.
     * Not sure if this method belongs under AssociationService, but
     * it is certainly a part of the data service.
     * 
     * @param array of File objects to add.
     * @throws ServiceException if errors are reported by handlers. The
     * stack trace information for this exception does not identify source
     * of the error, it is more a means of communicating errors.
     */
    public void addFiles(File[] files) throws ServiceException {
        Iterator i = handlerChain.iterator();
        StringBuffer errorReport = new StringBuffer();
        while (i.hasNext()) {
            AssociationHandler h = (AssociationHandler)i.next();
            try {
                h.addFiles(files);
            } catch (ServiceException e) {
                errorReport.append(e.toString()+"\n");
            }
        }
        if (errorReport.length() > 0) {
            throw new ServiceException(errorReport.toString());
        }
    }
    
    /**
     * Remove the listed files from the association model.
     * Not sure if this method belongs under AssociationService, but
     * it is certainly a part of the data service.
     * @param array file File objects to be removed.
     */
    public void removeFiles(File[] files) throws ServiceException {
        Iterator i = handlerChain.iterator();
        StringBuffer errorReport = new StringBuffer();
        while (i.hasNext()) {
            AssociationHandler h = (AssociationHandler)i.next();
            h.removeFiles(files);
        }
    }
    
    
    
    /**
     * Add the given handler to the chain of handlers.
     * If already present, the previous handler is removed and added again.
     * Additions are made to the end of the chain.
     * @param h the display handler to be added to the chain.
     */
    public void addAssociationHandler(AssociationHandler h) {
        removeAssociationHandler(h);
        handlerChain.add(h);
    }
    
    /**
     * Remove the given handler from the handler chain.
     * @param h
     */
    public void removeAssociationHandler(AssociationHandler h) {
        handlerChain.remove(h);
    }
}
