package org.eso.dfs.util;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * @author Nick Kornweibel
 *
 * The SharedList encapsulates (and to a large extent exposes) an ArrayList with the
 * added features of maintaining a list of SharedListListeners that are notified when the
 * SharedList content changes.
 * 
 * The SharedList is a set - not a list...badly named.
 * 
 * It is important to call removeListListener() when a listener is not longer
 * needed - failing to do this will result in the listener not being garbage collected as 
 * expected, because this list still holds a refence to the listener.
 * 
 * Note: the SharedList is NOT thread safe. This is the task of the calling application.
 */
public class SharedList {
    
        // the list of items, in sorted order
    private ArrayList list;
    
        // if true, the list is maintained in sorted
        // order, else false
    private boolean sortList = false;
    
        // the maximum number of items the list can hold
    private int maxCapacity = 20;

        // This is a list that maintains a history of the items selected.
        // index 0 is the latest item selected. When an item is selected, it
        // removed from its current index and placed at index 0. There are most
        // likely better implementations of this model, but the lists should be
        // of small capacity only and therefore should not be a performance problem.
    private List selectionHistory = new ArrayList();
    
        // List of listeners registered with this shared list.
    private List listeners = new ArrayList();
    
    /**
     * Constructor for SharedList.
     * @param initialCapacity
     */
    public SharedList(int initialCapacity) {
        list = new ArrayList(initialCapacity);
    }
    
    /**
     * Set list sorting on or off. If true, the
     * list will be sorted. This call will notify
     * listeners.
     * @param s
     */
    public void setSortList(boolean s) {
        sortList = s;
        sort();
        notifyListChanged();
    }
    
    /**
     * Return true if items are maintained in sorted order.
     * @return boolean
     */
    public boolean isSortList() {
        return sortList;
    }
    
    /**
     * Returns the maxCapacity.
     * @return int
     */
    public int getMaxCapacity() {
        return maxCapacity;
    }

    /**
     * Sets the maxCapacity. Does nothing if
     * max less than 0.
     * @param maxCapacity The maxCapacity to set
     */
    public void setMaxCapacity(int max) {
        if (max >= 0) {
            maxCapacity = max;
            trimToCapacity();
            sort();
        }
    }
    
    
    /**
     * Perform the various house keeping routines on the lists
     * after a selection has been made or the model has been 
     * modified.
     * If the newSelection is null or has a toString() value of "", the
     * newSelection is not integrated into the model.
     * @param newSelection the new item selected, possibly to be added to the model.
     */
    protected void updateList(Object newSelection) {
        if (newSelection != null && newSelection.toString().length() > 0) {
            boolean newItem = recordSelection(newSelection);
            if (newItem) {
                trimToCapacity();
                sort();
            }
        }
    }
    
    /**
     * Update the selectionHistory list with the
     * current selected item. This means moving the
     * selected item to index 0, if already in the list,
     * or inserting the item at index 0 if not in the list.
     * @return true if the item is a new item in the list, else false.
     */
    protected boolean recordSelection(Object newSelection) {
        int idx = selectionHistory.indexOf(newSelection);
        boolean newItem = idx < 0;
        if (idx > 0) {
            // item which is in the list but
            // not at idx 0
            selectionHistory.remove(idx);
            // place it at the front of the list
            // as the most recent selection.
            selectionHistory.add(0, newSelection);
        } else if (idx < 0) {
            // item which is not currently in the list
            // place the item at the front of the list
            // as the most recent selection.
            selectionHistory.add(0, newSelection);
        }
        if (newItem) {
            // add the item to the combobox model
            list.add(newSelection);
        }
        return newItem;
    }
    
    /**
     * Trim the lists to the current maxCapacity value.
     */
    protected void trimToCapacity() {
        while (selectionHistory.size() > maxCapacity) {
            Object removedItem = selectionHistory.remove(selectionHistory.size()-1);
            // remove the item from the combobox model.
            if (removedItem != null) {
                list.remove(removedItem);
            }
        }
    }
    
    protected void sort() {
        if (sortList) {
            Collections.sort(list, Collator.getInstance());
        }
    }

    /**
     * Constructor for SharedList.
     */
    public SharedList() {
        list = new ArrayList();
    }

    /**
     * Constructor for SharedList.
     * @param c
     */
    public SharedList(Collection c) {
        list = new ArrayList(c);
    }
   
    /**
     * Add a SharedListListener to the set of registered listeners.
     * Registered listeners will be notified when the contents of
     * this shared list have changed.
     * @param l
     */
    public void addListListener(SharedListListener l) {
        if (!listeners.contains(l)) {
            listeners.add(l);
        }
    }
    
    /**
     * Remove a SharedListListener from the set of registered
     * listeners.
     * @param l
     */
    public void removeListListener(SharedListListener l) {
        listeners.remove(l);
    }
    
    /**
     * Notify all registered SharedListListeners that the content
     * of the list has changed. This method should not be called
     * directly, rather it is called by the SharedList after 
     * any list edit methods are called.
     */
    protected void notifyListChanged() {
        Iterator i = listeners.iterator();
        while (i.hasNext()) {
            ((SharedListListener)i.next()).listChanged(this);
        }
    }
    
    /**
     * Add en element to the list and notify all listeners.
     * @param o
     */
    public void add(Object o) {
        updateList(o);
        notifyListChanged();
    }
    
    /**
     * Remove an element from the list and notify all listeners.
     * @param o Object to remove
     */
    public void remove(Object o) {
        list.remove(o);
        selectionHistory.remove(o);
        notifyListChanged();
    }
    
    /**
     * Remove an element from the list and notify all listeners.
     * @param i index of element to remove
     */
    public void remove(int i) {
        Object removedItem = list.get(i);
        list.remove(i);
        selectionHistory.remove(removedItem);
        notifyListChanged();
    }
    
    /**
     * Get the size of the shared list.
     * @return int the size of the shared list.
     */
    public int size() {
        return list.size();
    }
    
    /**
     * Get the element at the specified index (from 0).
     * @param i the index
     * @return Object the element at that index
     */
    public Object get(int i) {
        return list.get(i);
    }
  
    /**
     * Get an iterator to the elements in the shared list.
     * @return Iterator
     */  
    public Iterator iterator() {
        return list.iterator();
    }
}
