/**
 * @author N. Kornweibel
 * @version 1.0
 * A class defining the OBComponent of a DFSTextFile.
 */
package org.eso.gasgano.datamodel.gui;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import org.eso.gasgano.datamodel.DataModelChangeListener;
import org.eso.gasgano.datamodel.DataModelChangeNotifier;
import org.eso.gasgano.datamodel.DataModelEvent;
import org.eso.gasgano.datamodel.FrameInfo;
import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.datamodel.filesystem.DFSFile;
import org.eso.gasgano.datamodel.filesystem.DFSTextFile;
import org.eso.gasgano.gui.GUIUtils;
import org.eso.gasgano.properties.PropertyDB;

public class DFSTextFileOBComponent
    extends OBComponent
    implements DataModelChangeListener {

    private DFSTextFile myFile = null;
    private DataModelChangeNotifier eventSource = null;
    private static JLabel dataLabel =
        new JLabel(
            new ImageIcon(DFSTextFileOBComponent.class.getResource(GUIUtils.dataGif)),
            (int) JLabel.LEFT_ALIGNMENT);

    /**
     * @param address The file name/path of the text file.
     */
    public DFSTextFileOBComponent(String address) {
        myFile = new DFSTextFile(address);
        insertInDataModel();
        // ascii files are the result of pipe line reductions.
        setPipeProduct(true);
    }

    public DFSFile getSourceFile() {
        return myFile;
    }

    public int getStatus() {
        return myFile.getStatus();
    }

    /**
     * @return The OBComponent (A FitsFileOBComponent) which can be
     * used to get prog id, ob id info.
     * Or null if not appropriate component is located.
     */
    private OBComponent getIdentifyingComponent() {
        OBComponent rc = null;
        DFSDataModel dm = DFSDataModel.getDataModel();
        Vector dataSourceSet =
            dm.getObComponentByBaseName(myFile.getBaseName());
        if (dataSourceSet != null && dataSourceSet.isEmpty() == false) {
            Enumeration e = dataSourceSet.elements();
            while (rc == null && e.hasMoreElements()) {
                rc = (OBComponent) e.nextElement();
                if (rc instanceof FitsFileOBComponent == false) {
                    // we only want to find a FitsFile.
                    rc = null;
                }
            }
        }
        return rc;
    }

    /**
     * Adding a DFSTextFileOBComponent into a DataModel:
     * Search for an entry in the data model which has the
     * same base name as this file. If located, add this into the
     * DataModel using the prog and ob info for the located
     * component, and add this as a DataModelChange listener to 
     * that Component, so that the procedure is repeated if the 
     * data source changes.
     * If no file is located, add under "Unknown" catagories and add this
     * as a DataModelChangeListener to the DataModel, 
     * to repeat the procedure if new files are added.
     * If the component is current a part of the data model, and it is
     * determined that it must be moved, it will be removed, and then added.
     */
    private void insertInDataModel() {
        DFSDataModel dm = DFSDataModel.getDataModel();
        String progId = null;
        String obId = null;
        String telescopeId = null;
        if (!dm.passFilter(this)) {
            myFile.setStatus(DFSFile.FILE_FILTERED);
            return;
        }
        OBComponent dataSource = getIdentifyingComponent();
        DataModelChangeNotifier notifier = null;
        if (dataSource == null) {
            progId = DFSDataModel.UNKNOWN_PROGRAM_ID;
            obId = DFSDataModel.UNKNOWN_OBSERVATION_ID;
            telescopeId = DFSDataModel.UNKNOWN_TELESCOPE_ID;
            notifier = dm;
        } else {
            FrameInfo frameInfo = new FrameInfo();
            dm.provideFrameInfo((KeywordContainerOBComponent) dataSource, frameInfo);
            telescopeId = frameInfo.telescopeId;
            progId = frameInfo.programId;
            obId = frameInfo.obsId;

            notifier = dataSource;
        }
        if (storeProgram != null
            && storeProgram.equals(progId)
            && storeObservation != null
            && storeObservation.equals(obId)
            && storeTelescope != null
            && storeTelescope.equals(telescopeId)) {
            // we have not changed prog/ob/date...
        } else {
            // remove this from the model if in the model
            // and add this, if valid.
            if (eventSource != null) {
                storeProgram = null;
                storeObservation = null;
                storeTelescope = null;
                removeFromDataModel();
            }
            if (getStatus() == DFSFile.FILE_VALID) {
                String dir = myFile.getPath().getParent();
                dm.addObComponent(progId, obId, dir, telescopeId, this);
                storeProgram = (storeProgram == null) ? progId : storeProgram;
                storeObservation =
                    (storeObservation == null) ? obId : storeObservation;
                storeTelescope =
                    (storeTelescope == null) ? telescopeId : storeTelescope;
                registerToNotifier(notifier);
            }
        }
    }

    /**
     * @param n The DataModelChangeNotifier to which this is to be
     * registered. Register this as a listener to the DataChangeNotifier 
     * removing the old registration if required. This method assumes there 
     * is only one DataChangeNotifier regisitered to at one time.
     */
    private void registerToNotifier(DataModelChangeNotifier n) {
        if (eventSource != n) {
            if (eventSource != null) {
                eventSource.removeDataModelChangeListener(this);
            }
            if (n != null) {
                n.addDataModelChangeListener(this);
            }
            eventSource = n;
        }
    }

    /**
     * @return The status following the update().
     * Check the status of the file and return it.
     * if the status has changed, re-read the file and notify
     * the appropriate listeners if applicable.
     * @see org.eso.gasgano.DFSFile
     */
    public int update() {
        if (myFile.dataRefreshRequired()) {
            boolean inModel = (getStatus() == DFSFile.FILE_VALID);
            myFile.readData();
            insertInDataModel();
        }
        return getStatus();
    }

    /**
     * @param e The event to be processed.
     * Either the Component we used to get prog/ob info has
     * changed or we had no such Component and were in the Unknown group, 
     * and new files have been added to the DataModel,
     * either way, we need to remove
     * this from the data model and re-add this.
     */
    public void dataModelChanged(DataModelEvent e) {
        if (e.getEvent() == DataModelEvent.FILE_DELETED
            || e.getEvent() == DataModelEvent.FILE_ADDED
            || e.getEvent() == DataModelEvent.FILE_UPDATED) {
            if (PropertyDB.getInstance().isDebug(PropertyDB.debugEvent)) {
                System.out.println(getId() + " received dataModelChangeEvent.");
            }
            if (e.getFile() != this) {
                insertInDataModel();
            }
        }
    }

    /**
     * @return The JComponent which encapsulates the data of this file.
     */
    public JComponent getDisplayComponent() {
        JComponent rc = null;
        if (myFile.readFile() == DFSFile.FILE_NOT_FOUND) {

            if (!DFSDataModel.getDataModel().isUpdating()) {
                new Thread(new Runnable() {
                    public void run() {
                        DFSDataModel.getDataModel().update();
                    }
                }).start();
            }
            rc =
                new JLabel(
                    "The file has been slyly removed from the filesystem",
                    SwingConstants.CENTER);
            return rc;
        }

        rc = new DFSTextFileDisplay(toString(), myFile.getData());
        // free the memory (as soon as the Model is finished with it.
        myFile.freeData();
        return rc;
    }

    public JComponent getTreeCellComponent() {
        JLabel cell = dataLabel;
        cell.setText(new String(toString()));
        cell.setFont(getLabelFont());
        return cell;
    }

    /**
     * @return The id string which uniquely identifies this in the
     * DataModel.
     * NB: This is the original address with which the FitsFile was created,
     * not the resolved address returned from a File object.
     */
    public String getId() {
        return myFile.getAddress();
    }

    public void cleanup() {
        super.cleanup();
        // remove our notification registration.
        registerToNotifier(null);
    }

} // end of class DFSTextFileOBComponent

// end of file DFSTextFileOBComponent.java
