package org.eso.gasgano.dfspipeline;

import java.io.File;
import java.util.Properties;

/**
 * @author Nick Kornweibel
 *
 * A simple singleton of properties and methods for determining if Gasgano
 * is running in the "DFS Pipeline Environment".
 * 
 * The DFS Pipeline environment depends on the platform, various system
 * properties defining directories and the existence of these directories.
 * 
 * When all is ok the pipeline.dfs system property is set to true indicating
 * that Gasgano is running in the DFS Pipeline Environment.
 */
public class DFSPipelineEnvironment {

    /**
     * Singleton instance of this class.
     */
    private static  DFSPipelineEnvironment instance;
    
    private boolean pipelineEnvironment;

    private DFSPipelineEnvironment() {
        assessPipeline();
    }
    
    /**
     * this method stores in the System properties a value of
     * true or false, according to the presence or not of 
     * a pipeline reduction environment, including environment
     * variables pointing to existing directories and a 
     * matching operating system<p>
     * The value of the key <i>pipeline.dfs</i> will be checked by
     * TreeControlPanel.<p>
     * @see: TreeControlPanel
     */

    private void assessPipeline() {
        Properties environment = null;
        String redBlockDir = null;
        String reducedDataDir = null;
        String rawDataDir = null;
        // criteria to determine whether this machine/account is doing reduction
        // with a DFS pipeline
        boolean partlyConfigured = false;
        environment = System.getProperties();
        String osName = environment.getProperty("os.name");
        redBlockDir = (String) environment.getProperty("pipe.redblock");
        if (redBlockDir != null) {
            System.out.println("pipe.redblock: "+redBlockDir);
            partlyConfigured = true;
        }
        rawDataDir = (String) environment.getProperty("pipe.raw");
        if (rawDataDir != null) {
            System.out.println("pipe.raw: "+rawDataDir);
            partlyConfigured = true;
        }
        reducedDataDir = (String) environment.getProperty("pipe.reduced");
        if (reducedDataDir != null) {
            System.out.println("pipe.reduced: "+reducedDataDir);
            partlyConfigured = true;
        }

        System.getProperties().put("pipeline.dfs", "false");
        pipelineEnvironment = false;
        // ESO dual platform policy
        if ((osName.equals("HP-UX") || osName.equals("Solaris")
                || osName.equals("SunOS"))
            && redBlockDir != null
            && rawDataDir != null
            && reducedDataDir != null) {
            // additional checks for existence of dirs
            if (new File(rawDataDir).isDirectory()
                && new File(reducedDataDir).isDirectory()
                && new File(redBlockDir).isDirectory()) {
                // System.err.println("No reduction environment");
                System.getProperties().put("pipeline.dfs", "true");
                pipelineEnvironment = true;
                System.out.println("Running in DFS Pipeline Environment.");
            } else {
                System.out.println("A specified pipeline directory does not exist, cannot run in DFS Pipeline Environment.");
            }
        } else if (partlyConfigured){
            System.out.println("Incomplete or invalid DFS Pipeline configuration, cannot run in DFS Pipeline Environment.");
        }
    }

    /**
     * Returns the instance.
     * @return DFSPipelineEnvironment
     */
    public static DFSPipelineEnvironment getInstance() {
        if (instance == null) {
            instance = new DFSPipelineEnvironment();
        }
        return instance;
    }

    /**
     * Returns the pipelineEnvironment property. - the only really useful
     * method in this class.
     * This setting is also set in system properties as
     * pipeline.dfs ("true" or "false").
     * @return boolean
     */
    public boolean isPipelineEnvironment() {
        return pipelineEnvironment;
    }

}
