package org.eso.ohs.core.dbb.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.SQLException;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.rmi.DbbRemoteEngine;
import org.eso.ohs.core.dbb.rmi.DbbRemoteEngineImpl;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.rmi.OHSServer;

/**
 * @version $Id: DbbServer.java,v 1.5 2005/03/29 11:51:35 ddorigo Exp $
 *
 * This class implements a simple and generic application server for
 * the database browser client, taking all the configuration
 * parameters as arguments in the constructor.
 * 
 * @author C. Boarotto, ESO 10-Sep-2000
 * @author amchavan, 11-Oct-2001: re-implemented as a subclass of OHSServer.
 */
public class DbbServer
    extends OHSServer
    implements DbbEngineFactory {

private static final String rcsid = "$Id: DbbServer.java,v 1.5 2005/03/29 11:51:35 ddorigo Exp $";

private static Logger stdlog_ = 
    Logger.getLogger( DbbServer.class );

    /** JDBC url for the database driver */
    private String dbURL_;

    /** Database name to connect */
    private String dbName_;

    /** Database login */
    private String dbUserName_;

    /** Database password */
    private String dbPassword_;


    //----------------------------------------
    // Constructors
    //----------------------------------------

    /**
     * Creates DBB server object.
     *
     * @param name      Name to be used when contacting this server.
     *
     * @param port      Port number on which the server is
     *			going to listen for connection requests.
     *                  A value of zero means the system default.
     *
     *
     * @param db_url    Fully qualified URL of a JDBC server; for instance:
     *                  <BR>
     *                  <code>jdbc:sybase:Tds:pclinnt1.hq.eso.org:55500</code>
     *
     * @param db_name	Database name.
     *
     * @param db_user	User name to be used when logging in on 
     *			the database.
     *
     * @param db_passwd	Password to be used when logging in on 
     *			the database
     */
    public DbbServer( String name,
		      int    port,
		      String db_url,
		      String db_name,
		      String db_user,
		      String db_passwd )
	throws RemoteException, IOException, 
	       AlreadyBoundException, NotBoundException
    {
	super( name, port, null );
	startUp();
	dbURL_         = db_url;
	dbName_        = db_name;
	dbUserName_    = db_user;
	dbPassword_    = db_passwd;

	log( "dbURL_:      " + dbURL_ );
	log( "dbName_:     " + dbName_ );
	log( "dbUserName_: " + dbUserName_ );
	// log( "dbPassword_: " + dbPassword_ );
	log( "dbPassword_: " + "(not shown)" );

	try {
	    // Try to connect right away -- make sure
	    // we have the proper user ID and password
	    //----------------------------------------

	    Connection     conn = null;
	    ConnectionPool pool = null;
	    
	    pool = ConnectionPool.getPool();
	    conn = pool.acquireConnection( dbURL_, dbUserName_, dbPassword_ );
	    pool.releaseConnection( conn );
	}
	catch( SQLException e ) {
	    log( "Trying to connect:", e );
	    System.exit( 1 );
	}
	
	log( "Input args verified" );
    }


    //----------------------------------------
    // Public methods
    //----------------------------------------

    /**
     * Interface implementation.
     * This method sets up a database session, builds a remote engine
     * and returns its reference to the calling application.
     *
     * @return DbbRemoteEngine reference
     */
    public DbbRemoteEngine getEngine() throws SQLException, RemoteException {

	DbbSession session = new DbbSession(dbURL_,
					    dbName_,
					    dbUserName_,
					    dbPassword_);
	DbbSqlEngineImpl engine = new DbbSqlEngineImpl( session, false );
	return new DbbRemoteEngineImpl( engine, getLogger(), 
					getClientHostName() );
    }


    /**
     * Create and run a DbbServer taking the configuration parameters
     * from the array of the arguments.
     * <P>
     * args[0]: DBB server name, for instance <code>DBBserver</code><BR>
     * args[1]: DBB server port number, for instance <code>1220</code> 
     *          (default)<BR>
     * args[2]: JDBC server URL, for instance
     *          <code>jdbc:sybase:Tds:pclinnt1.hq.eso.org:55500</code><BR>
     * args[3]: DB name, for instance <code>opc63</code><BR>
     * args[4]: DB user ID, for instance <code>guest</code><BR>
     * args[5]: <em>optional</em>: DB user password, for instance 
     *          <code>guestPWD</code>. If this argument is not provided,
     *          the user will be prompted interactively for it.
     * <P>
     *
     * To run this server: <OL>
     * <LI> make sure your <code>CLASSPATH</code> environment variable is
     *      correctly set.
     * <LI> <code>cd ...</code> (just above the org/eso/ohs directory tree)
     * <LI> <code>java sun.rmi.registry.RegistryImpl &</code> (start the
     *      RMI registry)
     * <LI> <code>java org.eso.ohs.dbb.server.DbbServer ...</code> (see above).
     * </OL>
     */
    public static void main(String[] args) {
	
	DbbServer dbbs = null;
	String passwd  = null;
	String usage   = "Usage: DbbServer name port server_url " +
	                 "db_name db_user [db_passwd]";

	if( args.length < 5 ) {
	    stdlog_.error( usage );
	    System.exit(1);
	}

	try {
	    // Do we have a password?
	    if( args.length == 6 ) 
		// YES, use that 
		passwd = args[5];
	    else {
		// NO, prompt the user
		System.out.print( "Password: " );
		System.out.flush();

		BufferedReader in
		    = new BufferedReader( new InputStreamReader( System.in )); 
		passwd = in.readLine();
	    }

	    Integer p = new Integer(args[1]);
	    int port  = p.intValue();

	    dbbs = new DbbServer( args[0], port,    args[2], 
				  args[3], args[4], passwd );

	}
	catch (Exception e) {
	    stdlog_.warn("",e);
	    System.exit(1);
        }
    }
}
