package org.eso.ohs.core.dbb.sql;

// $Id: DbbSqlTable.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $

import java.io.Serializable;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.server.DbbSession;

/** 
 * DbbSqlTable class is conceived to store all the information
 * related to a database table, in order to identify it uniquely,
 * into a single object.
 *
 * @author Carlo Boarotto, ESO 15-Sep-1999
 */
public class DbbSqlTable implements Serializable {

private static final String rcsid = "$Id: DbbSqlTable.java,v 1.2 2004/04/08 12:09:51 tcanavan Exp $";
private static Logger stdlog_ = 
    Logger.getLogger( DbbSqlTable.class );

  /** Database name (also catalog name) */
  private String database_ = null;

  /** Database owner name (also schema name) */
  private String owner_ = null;

  /** Table name */
  private String table_ = null;

  //----------------------------------------
  // Constructors
  //----------------------------------------

  /** 
   * Build a table object just setting its database table name.
   *
   * @param table_name	Database table name
   */
  public DbbSqlTable(String table_name) {

      this(null, null, table_name);
  }


  /**  
   * Build a table object just setting the database name and the
   * database table name.
   *
   * @param db_name	Database table.
   * @param table_name Database table name.  
   */
  public DbbSqlTable(String db_name,
		     String table_name) {
      
      this(db_name, null, table_name);
  }


  /** 
   * Build a table object just setting the database name, the owner
   * name and the database table name.
   *
   * @param db_name	Database table.
   * @param owner_name	Database owner name.
   * @param table_name	Database table name.
   */
  public DbbSqlTable(String db_name,
		     String owner_name,
		     String table_name) {
      
      if(table_name == null || table_name.trim().equals(""))
	throw new IllegalArgumentException("bad table name: " + table_name);
      
      table_ = table_name;
      setDatabase(db_name);
      setOwner(owner_name);    
  }


  //----------------------------------------
  // Public methods
  //----------------------------------------

  /** @return The database name. */
  public String getDatabase() { return database_; }

  /** @return The table owner name. */
  public String getOwner() { return owner_; }

  /** @return The database table name. */
  public String getTable() { return table_; }


  /**
   * Set the database name.
   * @param db_name Database name.
   */
  public void setDatabase(String db_name) { 
    if(db_name != null && !db_name.trim().equals(""))
      database_ = db_name;
  }


  /**
   * Set the database owner name.
   * @param owner_name Database owner name.
   */
  public void setOwner(String owner_name) { 
    if(owner_name != null && !owner_name.trim().equals(""))
      owner_ = owner_name;
  }


  /**
   * Set the database and the owner names (if they are null) with the
   * default values got from the database session.
   *
   * @param db_session Database session. 
   */
  public void completeTable(DbbSession db_session) {
    if(database_ == null)
      database_ = db_session.getDefaultDatabase();
    if(owner_ == null)
      owner_ = db_session.getDefaultOwner();
  }


  /**
   * Redefine Object's method equals in order to make it a suitable
   * class of objects to be stored into stacks or vectors.<br>
   * Two DbbSqlTable are equals if they have the same table name
   *
   * @param obj	Object
   * @return Boolean value: true if it's equal, false otherwise.  
   */
  public boolean equals(Object obj) {

    // stdlog_.debug("Object <"+obj+">");
      if(obj == null)
	  return false;

    DbbSqlTable table_obj = (DbbSqlTable)obj;

    String tbl_1 = get();
    String tbl_2 = table_obj.get();
    // stdlog_.debug("Comparing <"+tbl_1+"> with <"+tbl_2+">");

    return tbl_1.equals(tbl_2);
  }


  /**
   * Redefine Object's method hashCode in order to make it a suitable
   * class of objects to be stored into hashtables.<br>
   *
   * @return The complete table name hashcode
   */
  public int hashCode() {
    String tbl = get();
    return tbl.hashCode();
  }


  /**
   * @return A String containing the complete table name
   *         (db.owner.table).
   */
  public String get() {
    
    String ret = "";

    if(database_ != null &&  owner_ != null) {

      ret = database_ + ".";
      ret = ret + owner_ + ".";
      ret = ret + table_;

    } else {

      if(database_ != null) {
	ret = database_ + "..";
	ret = ret + table_;
      } else
	ret = table_;
    }

    return ret;
  }


  /** 
   * Allow to dump the table information 
   * @return A String containing the complete table name
   *         (db.owner.table).
   */
  public String toString() { return this.get(); }


  //----------------------------------------
  // Debug methods
  //----------------------------------------

  public static void main(String[] args) {

    try {
      // test1
      DbbSqlTable t1 = new DbbSqlTable("opc63", "dbo", "eso_users");
      DbbSqlTable t2 = new DbbSqlTable("eso_users");

      stdlog_.debug("Table t1: <" + t1.get() + ">");
      stdlog_.debug("Table t2: <" + t2.get() + ">");
      stdlog_.debug("t1 compared with t2: <" + t1.equals(t2) + ">");
      stdlog_.debug("t2 compared with t1: <" + t2.equals(t1) + ">");

      // test2
      DbbSqlTable t3 = new DbbSqlTable("opc63", null, "eso_users");
      DbbSqlTable t4 = new DbbSqlTable("opc63", null, "eso_users");

      stdlog_.debug("Table t3: <" + t3.get() + ">");
      stdlog_.debug("Table t4: <" + t4.get() + ">");
      stdlog_.debug("t3 compared with t4: <" + t3.equals(t4) + ">");
      stdlog_.debug("t4 compared with t3: <" + t4.equals(t3) + ">");

      // test3
      DbbSqlTable t5 = new DbbSqlTable(null, "dbo", "eso_users");
      DbbSqlTable t6 = new DbbSqlTable(null, null, "eso_users");
    
      stdlog_.debug("Table t5: <" + t5.get() + ">");
      stdlog_.debug("Table t6: <" + t6.get() + ">");
      stdlog_.debug("t5 compared with t6: <" + t5.equals(t6) + ">");
      stdlog_.debug("t6 compared with t5: <" + t6.equals(t5) + ">");

      // test4 (1 & 3)
      stdlog_.debug("Table t2: <" + t2.get() + ">");
      stdlog_.debug("Table t6: <" + t6.get() + ">");
      stdlog_.debug("t2 compared with t6: <" + t2.equals(t6) + ">");
      stdlog_.debug("t6 compared with t2: <" + t6.equals(t2) + ">");
    }
    catch (Exception e) { 
      stdlog_.warn("",e); 
    }
  }  
}
