package org.eso.ohs.core.utilities;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * 
 * Class FileIOUtils
 * 
 * Static helper class, used to manipulate text and strings.
 * 
 * <h3>Usage</h3>
 * 
 * <pre>
 * 
 *  String myString = FileIOUtils.fileToString(File)
 *  &lt;pre&gt;
 * 
 *  @version $Revision: 1.13 $
 *  @author fsogni, May 3, 2005
 * 
 */

// $Id: FileIOUtils.java,v 1.13 2005/09/29 09:58:03 gdonino Exp $
public class FileIOUtils {

	private final static int DEFAULT_BUF_LEN = 64000;

	private static org.apache.log4j.Logger stdlog_ = org.apache.log4j.Logger
			.getLogger(FileIOUtils.class);

	static boolean overwriteFlag_ = false;

	public static void stringToFile(String s, String filename)
			throws IOException {

		BufferedWriter bw = new BufferedWriter(new FileWriter(filename));
		bw.write(s, 0, s.length());
		bw.close();

	}

	/**
	 * 
	 * @param s
	 *            input string
	 * @param filename
	 *            destination file
	 * @throws IOException
	 */
	public static void stringToFile(String s, File filename) throws IOException {

		BufferedWriter bw = new BufferedWriter(new FileWriter(filename));
		bw.write(s, 0, s.length());
		bw.close();

	}

	/**
	 * Read a file and return its contents as a String.
	 * 
	 * @param filename
	 *            Pathname of the file to read.
	 * @throws IOException
	 *             If something went wrong.
	 */
	public static String fileToString(String filename) throws IOException {
		File filePath = new File(filename);
		stdlog_.debug("Getting data from file " + filePath.getAbsolutePath());
		Reader r = new FileReader(filePath);
		return fileToString(r);
	}

	/**
	 * Read a file and return its contents as a String.
	 * 
	 * @param file
	 *            File to read.
	 * @throws IOException
	 *             If something went wrong.
	 */
	public static String fileToString(File file) throws IOException {
		Reader r = new FileReader(file);
		return fileToString(r);
	}

	/**
	 * Read a file from an InputStream and return its contents as a String.
	 * 
	 * @throws IOException
	 *             If something went wrong.
	 */
	// amchavan, 18-Mar-2004: added
	public static String fileToString(InputStream stream) throws IOException {
		Reader r = new InputStreamReader(stream);
		return fileToString(r);
	}

	/**
	 * Read a file from a Reader and return its contents as a String.
	 * 
	 * @throws IOException
	 *             If something went wrong.
	 */
	// amchavan, 18-Mar-2004: factored out from fileToString(String)
	public static String fileToString(Reader reader) throws IOException {

		BufferedReader br = new BufferedReader(reader);
		StringBuffer sb = new StringBuffer();
		char[] buff = new char[DEFAULT_BUF_LEN];
		while (br.ready()) {
			int nread = br.read(buff, 0, buff.length);
			if (nread <= 0) {
				break;
			}
			sb.append(buff, 0, nread);
		}
		br.close();
		return sb.toString();
	}

	/**
	 * Returs the canonical path of a file, in case of exception the absolute
	 * path is returned.
	 * 
	 * @param f
	 *            Input file
	 * @return path
	 */
	public static String getCanonicalPath(File f) {
		try {
			return (f.getCanonicalPath());
		} catch (IOException e) {
			return (f.getAbsolutePath());
		}
	}

	/**
	 * Copy file source to file destination. In case the destinations exists
	 * already, permission to overwrite is asked through standard input.
	 * 
	 * @param source_name
	 * @param dest_name
	 * @throws IOException
	 */
	public static void copyFile(String source_name, String dest_name)
			throws IOException {
		File source_file = new File(source_name);
		File destination_file = new File(dest_name);
		FileInputStream source = null;
		FileOutputStream destination = null;
		byte[] buffer;
		int bytes_read;

		if (source_file.isDirectory()) {
			//the source file is actually a directory, return silently
			return;
		}

		try {
			// First make sure the specified source file
			// exists, is a file, and is readable.
			if (!source_file.exists() || !source_file.isFile())
				throw new FileCopyException("FileCopy: no such source file: "
						+ source_name);
			if (!source_file.canRead())
				throw new FileCopyException("FileCopy: source file "
						+ "is unreadable: " + source_name);

			// If the destination exists, make sure it is a writeable file
			// and ask before overwriting it. If the destination doesn't
			// exist, make sure the directory exists and is writeable.
			if (destination_file.exists()) {
				if (destination_file.isFile()) {
					if (overwriteFlag_ == false) {
						//DataInputStream in = new DataInputStream(System.in);
						BufferedReader in = new BufferedReader(
								new InputStreamReader(System.in));
						String response;

						if (!destination_file.canWrite())
							throw new FileCopyException(
									"FileCopy: destination "
											+ "file is unwriteable: "
											+ dest_name);

						System.out.print("File " + dest_name
								+ " already exists.  Overwrite? (Y/N): ");
						System.out.flush();
						response = in.readLine();
						if (!response.equals("Y") && !response.equals("y"))
							throw new FileCopyException(
									"FileCopy: copy cancelled.");
					}
				} else
					throw new FileCopyException("FileCopy: destination "
							+ "is not a file: " + dest_name);
			} else {
				File parentdir = parent(destination_file);
				if (!parentdir.exists())
					throw new FileCopyException("FileCopy: destination "
							+ "directory doesn't exist: " + dest_name);
				if (!parentdir.canWrite())
					throw new FileCopyException("FileCopy: destination "
							+ "directory is unwriteable: " + dest_name);
			}

			// If we've gotten this far, then everything is okay; we can
			// copy the file.
			source = new FileInputStream(source_file);
			destination = new FileOutputStream(destination_file);
			buffer = new byte[1024];
			while (true) {
				bytes_read = source.read(buffer);
				if (bytes_read == -1)
					break;
				destination.write(buffer, 0, bytes_read);
			}
		}
		// No matter what happens, always close any streams we've
		finally {
			if (source != null)
				try {
					source.close();
				} catch (IOException e) {
					;
				}
			if (destination != null)
				try {
					destination.close();
				} catch (IOException e) {
					;
				}
		}
	}

	/**
	 * Copy file source to file destination. In case the destinations exists
	 * already, permission to overwrite is asked through standard input. <BR>
	 * If the parameter overwriteFlag is set to "true", the file destination (if
	 * existing) is overwritten without asking.
	 * 
	 * @param source_name
	 * @param dest_name
	 * @param overwriteFlag
	 * @throws IOException
	 */
	public static void copyFile(String source_name, String dest_name,
			boolean overwriteFlag) throws IOException {

		overwriteFlag_ = overwriteFlag;
		copyFile(source_name, dest_name);

	}

	/**
	 * Copy the files under source_directory to dest_directory. Subdirectories
	 * are NOT copied over. If existing files are overwritten.
	 * 
	 * 
	 * @param source_directory
	 * @param dest_directory
	 * @throws IOException
	 */
	public static void copyDir(String source_directory, String dest_directory)
			throws IOException {

		//Create a list of files...
		File source_dir = new File(source_directory);
		String[] source_files = source_dir.list();

		//..and copy them.
		for (int i = 0; i < source_files.length; i++) {
			String sourceFile = source_directory + File.separator
					+ source_files[i];
			String destinationFile = dest_directory + File.separator
					+ source_files[i];
			copyFile(sourceFile, destinationFile, true);
		}

	}

	
	/** Compresses the given files and sends back the result
	 * 
     * @param sourceFiles
     * @param tarFile
     * @return File
	 * @throws IOException
     */
    private static File zipFiles(File[] sourceFiles, String entryName, File tarFile) throws IOException {
        int read = 0;
        FileInputStream in;
        byte[] data = new byte[1024];
        
            ZipOutputStream out = new ZipOutputStream(new FileOutputStream(
                    tarFile));
            // Define the archive mode
            out.setMethod(ZipOutputStream.DEFLATED);
            // Compress files one by one.
            for (int i = 0; i < sourceFiles.length; i++) {
                // Define an entry for each file. This is how it looks in the .zip file
            	ZipEntry entry;
            	
            	if (entryName == null) {
            		entry = new ZipEntry(sourceFiles[i].getAbsolutePath());
            	} else {
            		entry = new ZipEntry(entryName + File.separator + sourceFiles[i].getName());
            	}
                in = new FileInputStream(sourceFiles[i]);
                out.putNextEntry(entry);
                // Write file
                while ((read = in.read(data, 0, 1024)) != -1) {
                    out.write(data, 0, read);
                }
                out.closeEntry();
                in.close();
            }
            out.close();
        
        return tarFile;
    }
	
	
	public static File zipFiles(String sourceDirectory, String entryName, File tarFile) throws IOException {
		
		//Create a list of files...
		File source_dir = new File(sourceDirectory);
		String[] source_files = source_dir.list();
	    File [] dest_files = new File[source_files.length];
		for (int i=0; i<source_files.length; i++) {
			dest_files[i] = new File(sourceDirectory + File.separator + source_files[i]);
		
		}
		File resultFile = zipFiles(dest_files, entryName, tarFile);
		
		return resultFile;
	}
    
	/** Compresses the given files and sends back the result
	 * 
     * @param sourceFiles
     * @param tarFile
     * @return File
	 * @throws IOException
     */
    public static File zipFiles(File[] sourceFiles, File tarFile) throws IOException {
    
    	File resultFile = zipFiles(sourceFiles, null, tarFile);
    	return resultFile;
    }
    
    
    
    /**
     * 
     * Deletes all files and subdirectories under dir.
     * Returns true if all deletions were successful.
     * If a deletion fails, the method stops attempting to delete and returns false.
     * 
     * @param source directory
     * @return boolean
     */
    public static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
    
        // The directory is now empty so delete it
        return dir.delete();
    }
    
    
    
	/**
	 * Read a file and put it into an array of bytes. The length of array must
	 * be specified.
	 * 
	 * @param file
	 *            input file
	 * @param bufLen
	 *            buffer length
	 * @return byte[]
	 * @throws IOException
	 */
	public static byte[] readBinaryFile(String file, int bufLen)
			throws IOException {
		InputStream in = null;
		byte[] out = new byte[0];
		in = new BufferedInputStream(new FileInputStream(file));
		// the length of a buffer can vary
		byte[] buf = new byte[bufLen];
		byte[] tmp = null;
		int len = 0;
		while ((len = in.read(buf, 0, bufLen)) != -1) {
			// extend array
			tmp = new byte[out.length + len];
			// copy data
			System.arraycopy(out, 0, tmp, 0, out.length);
			System.arraycopy(buf, 0, tmp, out.length, len);
			out = tmp;
			tmp = null;
		}
		return out;
	}

	/**
	 * Read a file and put it into an array of bytes. The lenght of the returned
	 * array is DEFAULT_BUF_LEN long
	 * 
	 * @param file
	 * @return byte[]
	 * @throws IOException
	 */
	public static byte[] readBinaryFile(String file) throws IOException {
		return readBinaryFile(file, DEFAULT_BUF_LEN);
	}

	/**
	 * Write a byte array to a file.
	 * 
	 * @param byteArray
	 * @param file
	 * @throws IOException
	 */
	public static void writeBinaryFile(byte[] byteArray, File file)
			throws IOException {
		FileOutputStream fileOutput = new FileOutputStream(file);
		fileOutput.write(byteArray);
		fileOutput.flush();
		fileOutput.close();
	}

	/**
	 * Convert an input stream into an array of bytes
	 * 
	 * @param in
	 * @param bufLen
	 * @return byte[]
	 * @throws IOException
	 */

	public static byte[] convertInputStreamToBinary(InputStream in, int bufLen)
			throws IOException {
		byte[] out = new byte[0];
		// the length of a buffer can vary
		byte[] buf = new byte[bufLen];
		byte[] tmp = null;
		int len = 0;
		while ((len = in.read(buf, 0, bufLen)) != -1) {
			// extend array
			tmp = new byte[out.length + len];
			// copy data
			System.arraycopy(out, 0, tmp, 0, out.length);
			System.arraycopy(buf, 0, tmp, out.length, len);
			out = tmp;
			tmp = null;
		}
		return out;
	}

	/**
	 * Convert an input stream into an array of bytes
	 * 
	 * @param in
	 * @param bufLen
	 * @return byte[]
	 * @throws IOException
	 */

	public static byte[] convertInputStreamToBinary(InputStream in)
			throws IOException {
		return convertInputStreamToBinary(in, DEFAULT_BUF_LEN);

	}

	// File.getParent() can return null when the file is specified without
	// a directory or is in the root directory.
	// This method handles those cases.
	private static File parent(File f) {
		String dirname = f.getParent();
		if (dirname == null) {
			if (f.isAbsolute())
				return new File(File.separator);
			else
				return new File(System.getProperty("user.dir"));
		}
		return new File(dirname);
	}

	/**
	 * Find a resource and return a stream to read it. <BR>
	 * The resource will be looked up first it the classpath: if it is not found
	 * there, it will be be looked up in the file system.
	 * 
	 * @param pathname
	 *            Pathname of the resource to find.
	 * @return An InputStream for the resource, if it could be opened.
	 * @throws IOException
	 *             If the stream could not be read.
	 */
	// amchavan, 18-Mar-2004: added
	public static InputStream getResourceStream(String pathname)
			throws IOException {

		InputStream s = null; // this is the stream we return

		// Look for the resource on the classpath,
		// using the best available class loader
		//---------------------------------------
		ClassLoader cl = FileIOUtils.class.getClassLoader();
		URL url = cl != null ? cl.getResource(pathname) : ClassLoader
				.getSystemResource(pathname);

		if (url != null) {
			s = url.openStream();
			return s;
		}

		// Look for the resource on the file system
		//-----------------------------------------
		File f = new File(pathname);
		s = new FileInputStream(f);

		return s;
	}
}

class FileCopyException extends IOException {
	public FileCopyException(String msg) {
		super(msg);
	}
}