package org.eso.ohs.persistence;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.evm.EvmExecutorImpl;

/**
 * EVMStorageMgr either verifys an OB before checkin or
 * solely verifys an OB from, for example EVMs, on  the
 * associated external media 
 * or with one at each end of an RMI connection between a client
 * program and the application server.
 * If an error is found an ObjectIOException is thrown.
 * 
 * <p>
 * $Id: EVMStorageMgr.java,v 1.16 2005/04/14 09:41:30 tcanavan Exp $
 * <br>
 * @author cmavros
 * <br>ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP AppServer.
 * 
 * @see AuthenticatingStorageMgr
 * @see StorageManager
 */
public class EVMStorageMgr extends DecoratorStorageManager {

    private static Logger stdlog_ =
       Logger.getLogger(EVMStorageMgr.class );

    /** Members used for Verification. */
    protected StringBuffer msgLog_;
    
    /**
     * In order to build a CalibrationBlock from a MultiObjectSpec, certain
     * phase1 data is required. This is only obtainable through this
     * StorageManager, which is passed into the constructor.
     */
    private StorageManager phase1DataSource_;

    public EVMStorageMgr(StorageManager stMgr, StorageManager phase1DataSource ) {
	   super(stMgr);
       phase1DataSource_ = phase1DataSource;
    }

    /**
     * Check in the specified objects in a single atomic operation.
     * 
     * @param	usos	Descriptions of each StorableObject to be checked in.
     * 
     * @return		array of error strings.
     *			If length = 0, no errors.
     */
    public void checkIn(MultiObjSpec[] moi)
				throws ObjectIOException {

       boolean passed = doVerify(moi);
       stdlog_.debug("EVMStorageMgr checkIn call to evm verification = " + passed);

       if(passed){
           super.checkIn(moi);
       }
       else{
           return;
       }
    }

    /**
     * Do a EVM verification test for the specified objects in a single atomic operation.
     *
     * @param   usos    Descriptions of each StorableObject to do EVM verification
     *
     * @return boolean if OB passed EVM test
     */
    public boolean verifyObject(MultiObjSpec[] moi)
				throws ObjectIOException {

       boolean passed = doVerify(moi);

       if (passed) {
           return super.verifyObject(moi);
        }
        else {
           return false;
        }
    }

    /**
     * Do a EVM verification test for the specified objects in a single atomic operation.
     *
     * @param   usos    Descriptions of each StorableObject to do EVM verification
     *
     * @return boolean if OB passed EVM test
     * 
     * NOTE: this call must be synchonrized as the EVM code and its script interpreters
     * is NOT thread-safe.
     */
    public synchronized boolean doVerify(MultiObjSpec[] moi)
                                throws ObjectIOException {
        try{
            setErrorMsg();
            int numErrs_ = 0;

            CalibrationBlock cb = null;

            for (int j = moi.length-1; j >= 0; j--) {
                StorableObject sobj = moi[j].obj.getObject();
                
                if(sobj instanceof CalibrationBlock){ 

                    cb =  StorageManager.MultiObjSpec.fromMultiObjSpecToCalBlock(moi, phase1DataSource_);

                    stdlog_.debug("EVMStorage id: "+cb.getDbaseId());
                    stdlog_.debug("EVMStorage name: "+cb.getName());
        
                    if (cb.getDbaseId() != 0){
                        String dbID = "" + Config.getCfg().uniqueToTableId(cb.getDbaseId());
                        for(int i = 0; i < (10 - dbID.length()); i++) {
                            dbID+=" ";
                        }
                        msgLog_.append("\nId: " 
                              + Config.getCfg().uniqueToTableId(cb.getDbaseId()) 
                              + " Name: " + cb.getName() + " - ");
                    } else {
                        msgLog_.append("\nName: " + cb.getName() + " - ");
                    }
                    // perform the stage 2 verification
                    String stage2VerifErrors = new String();
                    String stage2VerifMessages = new String();
                    StringBuffer header = new StringBuffer();
                    StringBuffer footer = new StringBuffer();

                    if (EvmExecutorImpl.evmConfigured()) {

                        boolean verifyPassed = true;
                        EvmExecutorImpl evm = EvmExecutorImpl.getInstance();

                        if (!evm.isInitialized()) {
                            stage2VerifErrors = "Verification not performed, "
                              +"verification module failed to initialize: "
                              +evm.getStatusString();
                        } else {
                            // Verify the OB
                            StringBuffer report = new StringBuffer();
                            verifyPassed = evm.verify((CalibrationBlock)cb, 
                                  report, header, footer);
                            stage2VerifErrors = report.toString();
                            if (stage2VerifErrors.length() > 0) {
                                stage2VerifErrors = "\n"+stage2VerifErrors+"\n";
                            }
                        }

                        // add the messages to our message log.
                        if (header.length() > 0) {
                            msgLog_.append(header.toString());
                        }
                        if (!verifyPassed){
                            msgLog_.append("FAILED");
                            msgLog_.append(stage2VerifErrors.toString());
                            numErrs_++;
                        } else {
                            msgLog_.append(" OK\n");
                        }
                        if (footer.length() > 0) {
                            msgLog_.append(footer.toString());
                        }
                    }
                }
            }
            if (numErrs_ > 0)
                throw new ObjectIOException(msgLog_.toString());
            else
	        return true;
         } catch(java.io.IOException e){
               //Exception thrown from fromMultiObjSpecToCalBlock()
               ErrorMessages.announceIOError(null, e);
               msgLog_.append(e);
        }
        return true;
     }

     /**
      * Return the error messages from the EVM verify
      * @return String error messages from EVM verify
      */
     public String getErrorMsg() {
        return msgLog_.toString();
     }

     /**
      * Set the error messages from the EVM verify
      */
     public void setErrorMsg() {
         msgLog_ = new StringBuffer();
     }
}


