package org.eso.ohs.persistence.dbase;

import java.io.IOException;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.ObjectFactory;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.SimpleStorableObject;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;

/**
 * A common superclass for database handlers of each kind
 * of BusinessObject.
 *
 * <p>
 * $Id: DbaseIOHandler.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public abstract class DbaseIOHandler {

    /** Standard log channel */
    protected static Logger stdlog_ =
        Logger.getLogger(DbaseIOHandler.class);

    /** UserId of the currently logged-in user */
    protected int userId_;

    /**
     * Set the user id for the handler
     * @param userID user id
     */
    public void setUserId(int userId) {
        userId_ = userId;
    }

    /**
     * Get the user id
     * @return user id
     */
    public int getUserId() {
        return userId_;
    }

    /**
     * Abstract method returns class for particular 
     * handler
     * @return Class
     */
    public abstract Class getManagedClass();

    /**
     * Create a new Instance of the type of BusinessObject managed
     * by the subclass of DbaseIOHandler.
     * 
     * @return a new BusinessObject instance.
     * @throws ObjectNotFoundException
     */
    protected BusinessObject newInstance() throws ObjectNotFoundException {
        Class cl = getManagedClass();
        return ObjectFactory.create(cl);
    }

    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * @exception ObjectIOException
     *		    an error occured during a read from the database
     * 
     * @throws IOException
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public abstract UnpackedStorableObject read(Statement stmt, long id)
        throws ObjectNotFoundException, ObjectIOException, IOException, SQLException;

    /**
          * Reads the object from the database, partially reassembling it.
          * References to other business objects are stored in other fields
          * of the array
          * 
          * @param conn  JDBC Connection for contacting the dbase
          * @param id    per-table id of the object to be retrieved.
          * @return      array in the format required by reassembleReferences()
          * 
          * @exception ObjectNotFoundException
          *          the requested object was not found in the database
          * @exception ObjectIOException
          *          an error occured during a read from the database
          * 
          * @throws IOException
     * @exception SQLException
          *          an error occurred during the dbase operation
          * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
          * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
          */
    public UnpackedStorableObject read(Connection conn, long id)
        throws ObjectNotFoundException, ObjectIOException, IOException, SQLException {
        Statement stmt = conn.createStatement();
        return read(stmt, id);
    }
    
    
    /**
     * Reads the object array from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbasa
     * @param whereClause The where clause for retreiving the DB.
     * @param otherTables String
     * @return      array in the format required by reassembleReferences()
     * 
     * @exception ObjectNotFoundException
     *          the requested object was not found in the database
     * @exception ObjectIOException
     *          an error occured during a read from the database
     * 
     * @throws IOException
     * @exception SQLException
     *          an error occurred during the dbase operation
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject[] read(
        Statement stmt,
        String whereClause,
        String otherTables)
        throws ObjectNotFoundException, ObjectIOException, IOException, SQLException {
        String msg = "Optional method, should be implemented in children only";
        if (true )
        throw new ObjectIOException(msg);
        return null;
    };


    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param conn  JDBC Connection for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @exception ObjectNotFoundException
     *          the requested object was not found in the database
     * @exception ObjectIOException
     *          an error occured during a read from the database
     * 
     * @throws IOException
     * @exception SQLException
     *          an error occurred during the dbase operation
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void delete(Connection conn, long id)
        throws ObjectNotFoundException, ObjectIOException, IOException, SQLException {
        Statement stmt = conn.createStatement();
        delete(stmt, id);
    } 
    
    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param conn  JDBC Connection for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @exception ObjectNotFoundException
     *          the requested object was not found in the database
     * @exception ObjectIOException
     *          an error occured during a read from the database
     * 
     * @throws IOException
     * @exception SQLException
     *          an error occurred during the dbase operation
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void delete(Connection conn, StorableObject obj)
        throws ObjectNotFoundException, ObjectIOException, IOException, SQLException {
        Statement stmt = conn.createStatement();
        delete(stmt, obj);
    }
    
    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @throws ObjectIOException
     * @exception SQLException  an error occurred during the dbase operation
     */
    public abstract void delete(Statement stmt, long id)
        throws ObjectIOException, SQLException;

    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @throws ObjectIOException
     * @exception SQLException  an error occurred during the dbase operation
     */
    public  void delete(Statement stmt, StorableObject obj)
        throws ObjectIOException, SQLException{
     throw new ObjectIOException (" delete operation not defined");  
    };





    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @throws ObjectIOException
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public abstract void write(
        Statement stmt,
        long id,
        UnpackedStorableObject obj)
        throws ObjectIOException, SQLException;

    /**
     * Writes the object to the database, for the first time.
     * 
     * @param con JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @throws ObjectIOException
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Connection con, long id, UnpackedStorableObject obj)
        throws ObjectIOException, SQLException {
            Statement stmt = con.createStatement();
            write(stmt, id,obj);

    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @throws ObjectIOException
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public abstract void update(
        Statement stmt,
        long id,
        UnpackedStorableObject obj)
        throws ObjectIOException, SQLException;

    /**
         * Updates an object that is already stored in the database.
         * 
         * @param conn JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
         * @param obj   an array: the first element is the BO to be stored.
         * @throws ObjectIOException
     * @exception SQLException  an error occurred during the dbase operation
         * 
         * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
         * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
         */
    public void update(Connection conn, long id, UnpackedStorableObject obj)
        throws ObjectIOException, SQLException {
        Statement stmt = conn.createStatement();
        update(stmt, id, obj);

    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param obj    element is the BO to be stored.
     * @throws ObjectIOException
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    protected void update(Statement stmt, StorableObject obj)
        throws ObjectIOException, SQLException {
        	String clName = this.getClass().getName();
        String msg = "The method update(Statement,StorableObject) is optional. \n This method should be implemented in class "+clName;
        stdlog_.error(msg);
        throw new ObjectIOException(msg);
    };

    /**
     * List the summaries for a particular business object
     * 
     * @param	stmt	JDBC Statement for querying the dbase.
     * 
     * @return Summary[]
     * @exception ObjectIOException
     *		    an error occurred during the dbase operation
     */
    public Summary[] listObjects(Statement stmt) throws ObjectIOException {
        return new Summary[] {
        };
    }

    /**
       * Connects to the database where objects of some class are stored.
       * 
       * @param   objType  The class we're interested in.
       *
       * @return  A JDBC Connection object, pointing to the database
       *          where objects of the <tt>objType</tt>> class are stored.
       * 
       * @exception ObjectIOException
       *            an error occured during an access to a business object
       *
       * <br>ChangeLog: amchavan, 15-Jan-2002: fixed bug, reformatted, fixed Javadoc.
       * @exception SQLException
       *            an error occurred during the dbase operation
       */
    private static Connection acquireConnection(Class objType)
        throws ObjectIOException, SQLException {
        Config cfg = Config.getCfg();
        String suffix = SimpleStorableObject.getSuffix(objType);
        String url = cfg.getUrl(suffix);
        String user = cfg.getDbUserName();
        String pwd = cfg.getDbPassword();
        // stdlog_.debug("The url: \""+url+"\" the user \""+ user+ "\" ");
        return ConnectionPool.getPool().acquireConnection(url, user, pwd);
    }

    /**
      * Calculate a unique id for local storage. 
      * 
      * @return Unique id
      * 
      * @exception ObjectIOException
      *         if there was an I/O error writing the object.
      */
    public static synchronized long getDbId() throws ObjectIOException {
        Connection con = null;
        CallableStatement cstmt = null;
        ResultSet rs = null;
        Config cfg = Config.getCfg();

        try {
            String obrepDbName =
                Config.getCfg().getDbName(ObservationBlock.getSuffix());
            con = DbaseIOHandler.acquireConnection(ObservationBlock.class);
            cstmt =
                con.prepareCall(
                    "{? = call " + obrepDbName + "..opc_get_key(?)}");

            cstmt.registerOutParameter(1, Types.INTEGER);

            cstmt.setString(2, "BUS_OBJ_ID");

            // The following calls match the code for
            //   the stored procedure 'opc_get_key'.
            // If you change one, you _must_ change the other.
            stdlog_.debug(">>>>>>>>>>>>> C Statement " + cstmt);
            cstmt.execute();
            cstmt.getMoreResults();
            cstmt.getMoreResults();
            rs = cstmt.getResultSet();
            if (rs != null) {
                rs.next();
                return rs.getLong(1);

            }
        } catch (SQLException e) {
            throw new ObjectIOException(e);
        } finally {
            if (con != null) {
                try {
                    if (cstmt != null) {
                        cstmt.close();
                    }
                } catch (SQLException e) {
                    e.printStackTrace();
                } finally {
                    ConnectionPool.getPool().releaseConnection(con);
                }
            }
        }

        throw new ObjectIOException("DbaseStorageMgr: could not generate a new database id");
    }
    
    /**
      * Calculate a unique id for local storage. 
      * 
      * @return Unique id
      * 
      * @exception ObjectIOException
      *         if there was an I/O error writing the object.
      */
    public static synchronized long getDbId(Connection con) throws ObjectIOException {
        return getId(con);
    }

    private static long getId(Connection con) throws ObjectIOException {
        CallableStatement cstmt = null;
        ResultSet rs = null;
        Config cfg = Config.getCfg();
        
        try {
            String obrepDbName =
                Config.getCfg().getDbName(ObservationBlock.getSuffix());
            cstmt =
                con.prepareCall(
                    "{? = call " + obrepDbName + "..opc_get_key(?)}");
        
            cstmt.registerOutParameter(1, Types.INTEGER);
        
            cstmt.setString(2, "BUS_OBJ_ID");
        
            // The following calls match the code for
            //   the stored procedure 'opc_get_key'.
            // If you change one, you _must_ change the other.
            stdlog_.debug(">>>>>>>>>>>>> C Statement " + cstmt);
            cstmt.execute();
            cstmt.getMoreResults();
            cstmt.getMoreResults();
            rs = cstmt.getResultSet();
            if (rs != null) {
                rs.next();
                return rs.getLong(1);
        
            }
        } catch (SQLException e) {
            throw new ObjectIOException(e);
        }         
        throw new ObjectIOException("DbaseStorageMgr: could not generate a new database id");
    }


}
