package org.eso.ohs.phase2.icdVcs;

import java.awt.Component;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * Auxiliary class to check that an OB/OD/TG unit is valid,
 * eg there is a Target (TG).  The run method for this class
 * runs in the EventQueue thread, so that it can perform Swing
 * operations, eg display dialog boxes for errors.
 * 
 * @author cmavros 2002/02/14, 1512/P2PP, add log4j
 * $Id: OBChecker.java,v 1.9 2004/04/08 12:09:53 tcanavan Exp $
 */
public class OBChecker implements Runnable {

    protected static Logger stdlog_ = Logger.getLogger(OBChecker.class);

    /** null or a reference to the last OB approved for transfer to BOB */
    private CalibrationBlock approvedBO_ = null;

    /** null or a reference to the last OB approved for transfer to BOB */
    private CalibrationBlock approvedBOClone_ = null;

    private ObjectManager objMgr = ObjectManager.getObjectManager();
    private IcdVcsCfg cfg = IcdVcsCfg.getCfg();
    private Component comp;

    private Media device_;

    /**
     * Default constructor.
     * 
     * @param fv	FolderGridView containing the selected OB
     */
    public OBChecker(Media device) {
        device_ = device;
    }

    public CalibrationBlock getOB() {
        return approvedBO_;
    }

    public CalibrationBlock getOBClone() {
        return approvedBOClone_;
    }

    /**
     * This method must only be run from the System Event Queue Thread !!
     * It manipulates the GUI and calls ObjectManager, neither of which
     * are Thread-safe.
     */
    public void run() {
        try {
            ObjectManager objMgr = ObjectManager.getObjectManager();

            comp = cfg.getContainer();

            approvedBO_ = null;
            approvedBOClone_ = null;

            CalibrationBlock[] cbs = grabCBs(1);
            CalibrationBlock[] checkedCBs = check(cbs);

            if (checkedCBs[0] != null) {
                cloneCB(checkedCBs[0]);
            }
        } catch (OutOfMemoryError e) {
            stdlog_.error(e);
            OutOfMemoryReport.showMessage();
            throw e;
        } catch (IOException e) {

        }
    }

    private void cloneCB(CalibrationBlock cb) throws IOException {
        DirectoryNode node = null;

        if (IcdVcsCfg.getCfg().isInVisitorMode()
            || (IcdVcsCfg.getCfg().isInEngineeringMode()
                && IcdVcsCfg.getCfg().isInCloneMode())) {
            try {

                node = objMgr.getRoot(Media.DBASE);

                CalibrationBlock newCB =
                    (CalibrationBlock) objMgr.deepCopy(Media.DBASE, node, cb);

                if (newCB != null) {
                    approvedBO_ = cb;
                    approvedBOClone_ = newCB;
                    // Set DbaseId for use in Unit Tests!
                    approvedBO_.setDbaseId(approvedBOClone_.getId());
                    newCB.setStatus(CalibrationBlock.INITIATED);
                    if (newCB.getType().equals("O")) {
                        ((ObservationBlock)newCB).setFindingCharts(null);
                    }
                    objMgr.putObHierarchy(newCB);
                } else {
                    approvedBO_ = cb;
                    approvedBOClone_ = cb;
                }

            } catch (ObjectIOException e) {
                stdlog_.error(e);
                ErrorMessages.announceIOError(comp, e);
            }
        } else {
            approvedBO_ = cb;
            approvedBOClone_ = cb;
        }

        // Both OBs sent to BOB take "Initiated" status.
        approvedBO_.setStatus(CalibrationBlock.INITIATED);
        approvedBOClone_.setStatus(CalibrationBlock.INITIATED);

        objMgr.queueObject(approvedBOClone_);
    }

    private CalibrationBlock[] grabCBs(int maxNumOBs) {
        long[] ids = cfg.getSelections(maxNumOBs);
        Vector busObjs = new Vector();

        if (!cfg.hasValidSelection()) {
            JOptionPane.showMessageDialog(
                comp,
                new Object[] {
                    "There is no selected business object",
                    "You must first select the observation block to execute." },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return new CalibrationBlock[] {
            };
        }

        for (int j = 0; j < ids.length; j++) {
            try {

                busObjs.addElement(
                    objMgr.getBusObj(device_, ids[j], cfg.getObjType(ids[j])));

            } catch (ObjectNotFoundException e) {
                stdlog_.error(e);
                ErrorMessages.announceNoObject(comp, e);
                return null; // Failure: return to caller
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(comp, e);
                return null; // Failure: return to caller
            }
        }

        CalibrationBlock[] cbs = new CalibrationBlock[busObjs.size()];
        busObjs.copyInto(cbs);

        return cbs;
    }

    private CalibrationBlock[] check(CalibrationBlock[] cbs) {
        CalibrationBlock[] goodCBs = new CalibrationBlock[cbs.length];
        int errcount = 0;

        System.arraycopy(cbs, 0, goodCBs, 0, cbs.length);

        for (int j = 0; j < cbs.length; j++) {
            if (!cbs[j].verify().equals("")) {
                goodCBs[j] = null;
                errcount++;
            }
        }

        if (errcount > 0) {
            int result =
                JOptionPane.showOptionDialog(
                    comp,
                    new Object[] {
                        (errcount == 1
                            ? "One error was found:"
                            : (errcount + " errors were found:")),
                        " ",
                        "Should the observation request(s) with errors",
                        "also be sent to the telescope software,",
                        "despite any errors?" },
                    "Error",
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.ERROR_MESSAGE,
                    null,
                    null,
                    "No");

            if (result == JOptionPane.YES_OPTION) {
                return cbs;
            } else {
                return goodCBs;
            }
        }

        return cbs;
    }

}
