package org.eso.ohs.phase2.visibility;

/**
 * AirMass class calculates airmass of the target
 * using the following SLALIB routines are called:
 *
 * slaZd - calculate the zenith angle
 * slaAirmas - calculate the airmass
 * slaDe2h - target eleveation
 *
 *
 * @author C. Mavros for ESO
 * @version 25 July 2001
 */

import org.apache.log4j.Logger;

public class AirMass extends JNIWrapper{

   /** 
    *  Standard log channel 
    * <br>ChangeLog: cmavros 21-12-01 added
    */
   private static Logger stdlog_ =
       Logger.getLogger( AirMass.class );

    /**
    * Longitutde of Observatory in radians
    */
    private static double PLONG = 1.228768; 

    /**
    * Latitude of Observatory in radians
    */
    private static double PLAT = -0.429802;

    /**
    * Elevation of Observatory
    */
    private static final double PELEVSEA = 2635.43;
    private static final double LELEVSEA = 2347.;

    /**
    * 180/pi  Radians to degrees 
    */
    private static final double R2D = 57.2957795130823;

    /**
    * Equatorial radius of earth, meters 
    */
    private static final double EQUAT_RAD = 6378137.0;


    /**
     * Note: this value is not only used for calculations, it is also
     * used in graphic presentation of the data - altering this value 
     * must be done keeping in mind that re-working of the display plots
     * may also be required.....do you really want that?????
     */
    public static final double NOT_VIS = 2.5;

    /**
     * calculateAirMass calls the getAirMass method.
     *
     * If the target is below the horizon or  
     * if the calculated value is less then zero the 
     * NOT_VIS value is returned. 
     *
     * @param jDate Julian Date of OB 
     * @param lst the sidereal time of the Julian Date
     * @param ra of target in radians 
     * @param dec of target in radians 
     * @param lat of observatory 
     * @return the airmass 
     */

    public static double calculateAirMass(double jdate, double lst,
      double targetRA, double targetDEC, double lat){

        double airMassValue = getAirMass(jdate, lst, targetRA, targetDEC, lat);

        /** Calculate if target is below the horizon */
        double horiz = Math.sqrt(2. * PELEVSEA / EQUAT_RAD) * R2D;
        double el = getEl(jdate, lst, targetRA, targetDEC, lat);

        if (el < -(horiz)){
            //Target is below the horizon
            airMassValue = NOT_VIS;
            /*
            stdlog_.info("visiplot.Visiplot.getAirMassValue - The target is below the horizon");
            stdlog_.info("visiplot.Visiplot.getAirMassValue Horizon         = " + horiz);
            stdlog_.info("visiplot.Visiplot.getAirMassValue Target horizon  = " + el);
            */
        }
        else if (el < 1.0){
            //Target is very low
            //stdlog_.info("visiplot.Visiplot.getAirMassValue Target very low = " + el);
            airMassValue = NOT_VIS;
        }
        else{
            if (airMassValue <= 0. ){
                stdlog_.info("visiplot.Visiplot.getAirMassValue - The target is not visible");
                airMassValue = NOT_VIS;
            }
            if (airMassValue > NOT_VIS ){
                //stdlog_.info("visiplot.Visiplot.getAirMassValue - " + 
                //  "Airmass to beyond the telescope's operational range");
                airMassValue = NOT_VIS;
            }
        }
        return(airMassValue);   
    }

    /**
     * getAirMass calculates the Airmass by 
     * calling the function slaZd for the zenith
     * angle and then the slaAirmas for the airmass.
     *
     * Note: Included is an another calculation for airmass
     * taken for the Skycalc program to use as a check
     * if neccessary.
     *
     * @param julian Date
     * @param local sidereal time
     * @param ra of the target
     * @param dec of the target
     * @param latitude of the Observatory
     * @return airmass value 
     */
    public static double getAirMass(double jDate, double lst, double ra, 
      double dec, double lat){

        double airmass = 0.0;
        double zenith = 0.0;


        /** Hour Angle - The angle between an observer's meridian and the
         * hour circle on which some celestial body lies.
         */
        double HA = lst - ra;

        zenith = jnislaZenith(HA, dec, lat);
        airmass = jnislaAirmass(zenith);

        /* Sanity check calculation for Airmass
           double checkAirmass = 1/(Math.sin(lat) * Math.sin(dec) + 
                     Math.cos(lat) * Math.cos(dec) * Math.cos(HA)); 
           stdlog_.info ("visiplot.Airmass.getAirMass airmass: "+checkAirmass);
        */

        /* Sanity check calculation for elevation in the method getEl()
           stdlog_.info ("visiplot.Airmass.getAirMass TEST : "+(90-(zenith*R2D))); 
        */

        /*
         stdlog_.info("visiplot.Airmass.getAirMass julian Date: "+jDate); 
         stdlog_.info("visiplot.Airmass.getAirMass lst: "+lst);
         stdlog_.info("visiplot.Airmass.getAirMass lat: "+lat);
         stdlog_.info("visiplot.Airmass.getAirMass HA: "+HA);
         stdlog_.info("visiplot.Airmass.getAirMass zenith: "+zenith);
         stdlog_.info("visiplot.Airmass.getAirMass airmass: "+airmass);
        */
        return(airmass);   
    }

    /**
     * getEl calls slaDe2h to
     * transform the HA and Dec into Azimuth and Elevation.
     * returning the elevation of a target
     *
     * Note: Included is an another calculation for sun angle
     * taken for the Skycalc program to use as a check
     * if neccessary.
     *
     * @param julian Date
     * @param local sidereal time
     * @param ra of the target
     * @param dec of the target
     * @param latitude of the Observatory
     * @return elevation of the object in degrees
     */
    private static double getEl(double jDate, double lst, double ra, 
      double dec, double lat){

        double rdpDe2h[] = new double[2];

        /** Hour Angle - The angle between an observer's meridian and the
         * hour circle on which some celestial body lies.
         */
        double HA = lst - ra;

        rdpDe2h = jnislaDe2h(HA, dec, lat);
        double el = rdpDe2h[1]*R2D;
        //stdlog_.info ("visiplot.Airmass.getEl altitude: "+ el);

        /* Sanity check calculation Sun twilight angle
           double checkEl = Math.asin(Math.sin(lat) * Math.sin(dec) + 
                            Math.cos(lat) * Math.cos(dec) * Math.cos(HA)); 
           stdlog_.info ("visiplot.Airmass.getAltit altitude: "+ checkEL*R2D);
        */

        return(el);
    }
}
