package org.eso.ohs.phase2.visibility;

/**
 * A PCFPlotterFilter extension for plotting air mass.
 * The class uses a set of discrete values to round inputs before storing.
 * The discrete value set is set in the constructor of the object.
 *
 * A note on the rounding used in this class:
 * Any value passed in the input() method which is less than the
 * smallest value in values[] will be stored as the smallest value, 
 * likewise for any value passed in input[] that is greater than the
 * largest value in values[] will be stored as the largest value.
 *
 * @author Nick Kornweibel for ESO.
 */

import java.util.Vector;

public class AirMassFilter extends PCFPlotterFilter {

    double[] values_ = null;
    double[] thresholds_ = null;
    double constraint_ = 0.0;

    public static  double defaultOutOfConstraintAirmass = AirMass.NOT_VIS;

    /**
     * Initialize an AirMassFilter object to use the values in the double
     * array.A note on rounding the values:
     * Any value passed in the input() method which is less than the
     * smallest value in values[] will be stored as the smallest value, 
     * likewise for any value passed in input[] that is greater than the
     * largest value in values[] will be stored as the largest value.
     *
     * @param array of values representing the discrete values. All -ve
     * values are set to 0.0.
     */

    public AirMassFilter(double values[]) {
        this(values, 0.0);
    }

    public AirMassFilter(double values[], double constraint) {
        // set the constraint
        setConstraint(constraint);
        // add the values in a sorted list, throwing out duplicates.
        Vector valueVec = new Vector(values.length);
        for (int i = 0; i < values.length; i++) {
            double val = values[i];
            if (val < 0.0) {
                val = 0.0;
            }
            for (int v = 0; v <= valueVec.size(); v++) {
                if (v == valueVec.size()) {
                    valueVec.insertElementAt(new Double(val), v);
                    break;
                } else {
                    double elem = ((Double)valueVec.elementAt(v)).doubleValue();
                    // throw out duplicates.
                    if (elem == val) {
                        break;
                    } else if (val < elem) {
                        valueVec.insertElementAt(new Double(val), v);
                        break;
                    }
                }
            }
        }
        // copy the valueVec into the values_ array
        values_ = new double[valueVec.size()];
        for (int i = 0; i < values_.length; i++) {
            values_[i] = ((Double)valueVec.elementAt(i)).doubleValue();
        }

        // calculate the value thresholds for use in rounding
        if (values_.length == 0) {
            thresholds_ = new double[0];
        } else {
            thresholds_ = new double[values_.length-1];
        }
        for (int i = 1; i < values_.length; i++) {
            double threshold = (values_[i]+values_[i-1])/2.0;
            thresholds_[i-1] = threshold;
        }
    }

    /**
     * Set the airmass constraint. All values greater than this constraint
     * are set to an airmass of defaultOutOfConstraintAirmass.
     *
     * @param the constraint airmass. 0.0 indicates no constraint is to be used.
     */

    public void setConstraint(double con) {
        constraint_ = con;
    }

    /**
     * @return the airmass constraint - value greater than this value
     * are set to defaultOutOfConstraintAirmass.
     */

    public double getConstraint() {
        return constraint_;
    }

    /**
     * Round the values to the nearest discrete value.
     * If there are no values set, the original value is returned.
     */

    public double input(PCFInterval interval)
            throws PCFPlotterFilter.ValueRejectedException {
         double rc = interval.getValue();
         double val = rc;
         
         for (int i = 0; i < values_.length; i++) {
             if (i > 0 && val <= thresholds_[i-1]) {
                 break;
             }
             rc = values_[i];
         }

         // confine the value according to the constraint.
         if (constraint_ != 0.0 && rc > constraint_) {
             rc = defaultOutOfConstraintAirmass;
         }
         return rc;
    }
}
