package org.eso.ohs.phase2.visibility;

/**
 * The class is initialized with a PCFPlotterFilter object that can
 * maniplate or reject values prior to storage in the PCF.
 *
 * If null is passed for the PCFPlotterFilter object, an instance of
 * PCFPlotterFilter is used.
 *
 * Filters may be swapped after creation using the setFilter() method.
 * 
 * @author Nick Kornweibel for ESO.
 */

import java.util.Date;

public class PCFPlotter extends PCF {

    private PCFPlotterFilter filter_ = null;

    /**
     * If null is passed for the PCFPlotterFilter object, an instance of
     * PCFPlotterFilter is used.
     *
     * @param the plotter filter object used to modify inputs to the PCF.
     */

    public PCFPlotter(PCFPlotterFilter filter) {
        init(DefaultValue, filter);
    }

    /**
     * Create a plotter with the specified default value and filter.
     * note the default value may be modified before setting in the pcf, 
     * however if the filter throws a ValueRejectedException, it is ignored
     * and the value stored. This is becase the PCF requires a default
     * value upon construction.
     */

    public PCFPlotter(double defVal, PCFPlotterFilter filter) {
        init(defVal, filter);
    }

    /**
     * Initialize call - only to be called by the constructors.
     */

    private void init(double defVal, PCFPlotterFilter filter) {
        setFilter(filter);
        try {
        defVal = filter.input(
                new PCFInterval(NegativeInfinity, PositiveInfinity, defVal));
        } catch (PCFPlotterFilter.ValueRejectedException e) {
            // exception is ignored - see method header for details.
        }
        // replace the current first element.
        pcf_.setElementAt(new PCFPoint(NegativeInfinity, defVal), 0);
    }

    /**
     * @return the filter object currently in use.
     */

    public PCFPlotterFilter getFilter() {
        return filter_;
    }

    /**
     * Replace/set the filter in the PCFPlotter.
     * @param the new filter to be used (does not apply to currently
     * saved data in the PCF).
     */

    public void setFilter(PCFPlotterFilter filter) {
        if (filter == null) {
            filter_ = new PCFPlotterFilter();
        } else {
            filter_ = filter;
        }
    }

    /**
     * Add an interval to the PCF - see PCF.java for details.
     */

    public boolean addInterval(Date startTime, Date endTime, double value) {
        boolean rc;
        try {
            value = filter_.input(new PCFInterval(startTime, endTime, value));
            rc = super.addInterval(startTime, endTime, value);
        } catch (PCFPlotterFilter.ValueRejectedException e) {
            rc = false;
        }
        return rc;
    }
}
