package org.eso.ohs.phase2.visiplot;

/**
 * Reporitory for Visibility Plot controls used for the display and calculation
 * of visibility plots.
 *
 * The VisiPlotOptions class stores all data as name and value pairs, and
 * can save/read data in from config files of PAF format.
 *
 * @author N. Kornweibel for ESO
 * <br>ChangeLog: cmavros 26/09/01 Add Time Interval menu option 
 */

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.NotASCIIException;
import org.eso.ohs.core.utilities.Paramfile;


public class VisiPlotOptions {

    /**
     * Standard log channel
     * <br>ChangeLog: cmavros 21-12-01 added
     */
    private static Logger stdlog_
            = Logger.getLogger( VisiPlotOptions.class );
    private Hashtable store_ = new Hashtable();

    private static final String header_ =
        "OHS software Visibility Plot configuration preferences";

    private static final long TwelveHoursInMilliseconds = 12 * 60 * 60 * 1000; 
    // Basic input data
    // The following two keys are needed but are not
    // saved in settings files, or read in from settings files.
    // If you add to these keys, remember to add to the setDefault()
    // method to set the default setting for this key.
    public static String startTimeKey         = "VISIPLOT.START.TIME";
    public static String endTimeKey           = "VISIPLOT.END.TIME";
    // group identical plots
    public static String groupPlotKey         = "VISIPLOT.PLOT.GROUP.OBS";
    // Sort plots by
    public static String sortPlotKey          = "VISIPLOT.PLOT.SORTBY";

    // Printer name
    public static String printerName          = "VISIPLOT.PRINTER";

    // Plot constraints
    public static String airMassConKey        = "VISIPLOT.AIR.MASS";
    public static String airMassConEnabledKey = "VISIPLOT.AIR.MASS.ENABLED";
    public static String moonDistConKey       = "VISIPLOT.MOON.DIST";
    public static String moonDistConEnabledKey = "VISIPLOT.MOON.DIST.ENABLED";
    public static String fliConKey            = "VISIPLOT.FLI";
    public static String fliConEnabledKey     = "VISIPLOT.FLI.ENABLED";
    public static String obexecConKey         = "VISIPLOT.OBEXEC.TIME.ENABLED";
    public static String obtimeIntervalConKey = "VISIPLOT.OBTIME.INTERVAL.ENABLED";
    public static String obSiderealTimeIntervalConKey = "VISIPLOT.OB.ST.TIME.INTERVAL.ENABLED";
    public static String sunConKey            = "VISIPLOT.SUN";
    public static String sunConEnabledKey     = "VISIPLOT.SUN.ENABLED";
    public static String zenithConKey 		  = "VISIPLOT.ZENITH.ENABLED";

    //Chart colors
    public static String airMassLineColor = "VISIPLOT.AIR.MASS.LINE.COLOR";
    public static String airMassAreaColor = "VISIPLOT.AIR.MASS.AREA.COLOR";
    public static String airMassLabelsColor = "VISIPLOT.AIR.MASS.LABELS.COLOR";
    public static String moonDistLineColor = "VISIPLOT.MOON.DIST.LINE.COLOR";
    public static String moonDistAreaColor = "VISIPLOT.MOON.DIST.AREA.COLOR";
    public static String moonDistLabelsColor = "VISIPLOT.MOON.DIST.LABELS.COLOR";
    public static String fliLineColor = "VISIPLOT.FLI.LINE.COLOR";
    public static String fliAreaColor = "VISIPLOT.FLI.AREA.COLOR";
    public static String fliLabelsColor = "VISIPLOT.FLI.LABELS.COLOR";
    public static String obexecLineColor = "VISIPLOT.OBEXEC.LINE.COLOR";
    public static String obexecAreaColor = "VISIPLOT.OBEXEC.AREA.COLOR";
    public static String obexecLabelsColor = "VISIPLOT.OBEXEC.LABELS.COLOR";
    public static String obSiderealTimeIntervalLineColor = "VISIPLOT.OB.ST.TIME.INTERVAl.LINE.COLOR";
    public static String obSiderealTimeIntervalAreaColor = "VISIPLOT.OB.ST.TIME.INTERVAl.AREA.COLOR";
    public static String obSiderealTimeIntervalLabelsColor = "VISIPLOT.OB.ST.TIME.INTERVAl.LABELS.COLOR";
    public static String sunLineColor = "VISIPLOT.SUN.LINE.COLOR";
    public static String sunAreaColor = "VISIPLOT.SUN.AREA.COLOR";
    public static String sunLabelsColor = "VISIPLOT.SUN.LABELS.COLOR";
    public static String timeLineColor = "VISIPLOT.TIME.LINE.COLOR";
    public static String timeAreaColor = "VISIPLOT.TIME.AREA.COLOR";
    public static String timeLabelsColor = "VISIPLOT.TIME.LABELS.COLOR";
    
    
    // observatory name:
    // Not needed.
    // public static String observatoryNameKey   = "VISIPLOT.OBSERVATORY.NAME";
    // SLALIB (or other astromonical library)
    // query time interval - in minutes

    public static String dataQueryIntKey      = "VISIPLOT.QUERY.INTERVAL";

    // plot header info - all either "T" or "F"
    public static String reportSeeingKey      = "VISIPLOT.REPORT.SEEING";
    public static String reportFLIKey         = "VISIPLOT.REPORT.FLI";
    public static String reportMoonDistKey    = "VISIPLOT.REPORT.MOONDIST";
    public static String reportSkyTransKey    = "VISIPLOT.REPORT.SKY";
    public static String reportAirMassKey     = "VISIPLOT.REPORT.AIRMASS";
    public static String reportRankKey        = "VISIPLOT.REPORT.RANK";
    public static String reportProgIdKey      = "VISIPLOT.REPORT.PROGID";
    public static String reportOBIdKey        = "VISIPLOT.REPORT.OBID";
    public static String reportRAKey          = "VISIPLOT.REPORT.RA";
    public static String reportDecKey         = "VISIPLOT.REPORT.DEC";
    public static String reportExecTimeKey    = "VISIPLOT.REPORT.EXECTIME";
    public static String reportStartTimeKey   = "VISIPLOT.REPORT.STARTTIME";

    // Sort options:
    // string values of constraints which can be used to
    // sort the visibility plots.
    public static String sortByProgid      = "program id";
    public static String sortByObid        = "OB-Id";
    public static String sortByRank        = "rank class";
    public static String sortBySeeing      = "seeing";
    public static String sortByFli         = "frac lunar illum.";
    public static String sortByMoondist    = "moon distance";
    public static String sortBySkytrans    = "sky transp.";
    public static String defaultSortOrder  = sortByProgid;

    public static String defaultSunCon     = "-18";

    public static int defaultPrecision    = 10;

    public static String useOBValue = "read from OB";

    /**
     * Create a VisiPlotOptions object, with no options stored.
     */

    public VisiPlotOptions() {
        setDefaults();
    }

    /**
     * Create a VisiPlotOptions object reading the input in PAF format
     * from the Reader object. Note the defaults settings are set
     * before over riding them from information from the PAF file.
     * @param the reader object containing the paf data for this.
     */

    public VisiPlotOptions(Reader in) throws IOException {
        setDefaults();
        readFrom(in);
    }

    /**
     * Update the store  with key/value pairs read from the
     * reader object parameter. All PAF parameters read are stored,
     * regardless of whether they are relevant to the options object or
     * not. Note that if start time or end time kezs are read, they
     * are ignored.
     *
     * @param the reader object containing the PAF format text parameters
     * @throws IOException if the reader format is not PAF format
     */

    public void readFrom(Reader in) throws IOException {
        Paramfile paf = null;
        try {
            // read the file, up to 10000 lines.
            paf = new Paramfile(in, 10000);
        } catch (NotASCIIException e) {
            throw new IOException("Input format error: "
                    +e.getMessage());
        }
        String[] idList = paf.getKeywords();
        for(int i = 0; i < idList.length ; i++) {
            String key = idList[i];
            String value = paf.getValue(key);
            if (key.equals(startTimeKey) || key.equals(endTimeKey)) {
                continue;
            }
            store_.put(key.toUpperCase(), value);
        }
    }

    /**
     * Write the store out, in PAF format, to the Write object passed
     * in the call. Note that start time and end time keys are not saved.
     *
     * @param the output writer to be used to write the PAF output store
     * @throws IOException if the write failed.
     */

    public void writeTo(Writer out) throws IOException {
        Paramfile paramfile = new Paramfile();
        for (Enumeration en = store_.keys(); en.hasMoreElements(); ) {
            String key = (String)en.nextElement();
            if (key.equals(startTimeKey) || key.equals(endTimeKey)) {
                // these keys are not save to file.
                continue;
            }
            String value = (String)store_.get(key);
            paramfile.put(key, value);
        }
        paramfile.writeTo(out, header_);
    }

    /**
     * Get an option value accoring to the key provided
     * Search is case in-sensitive.
     *
     * @param the name of the key to return (case insensitive).
     * @return the string value of the key, or null if not found
     */

    public String getValue(String key) {
        return (String)store_.get(key.toUpperCase());
    }
	
    /**
     * @return true if the value returned is "T" or "t", else false.
     */

    public boolean isEnabled(String key) {
        String val = getValue(key);
        return (val != null && (val.equals("T") || val.equals("t")));
    }
	
    /**
     * @return the double value of a value string, or throw a number
     * format exception if it can not be parsed.
     * @throws NumberFormatException if the value is null or not a number.
     */

    public double getDoubleValue(String key) throws NumberFormatException {
        String val = getValue(key);
        if (val == null) {
            throw new NumberFormatException("null value");
        }
        return Double.valueOf(val).doubleValue();
    }
	
    /**
     * @return the integer value of a value string, or throw a number
     * format exception if it can not be parsed.
     * @throws NumberFormatException if the value is null or not a number.
     */

    public int getIntValue(String key) throws NumberFormatException {
        String val = getValue(key);
        if (val == null) {
            throw new NumberFormatException("null value");
        }
        return Integer.parseInt(val);
    }

    /**
     * @return a Date object built from the ISO_DATE format date string value
     * or null if the string is not ISO_DATE format.
     */

    public Date getISODate(String key) {
        Date rc = null;
        String dateStr = getValue(key);
        if (dateStr != null) {
            SimpleDateFormat formatter
                = new SimpleDateFormat (Convert.ISO_DATE_FORMAT);
            rc = formatter.parse(dateStr, new ParsePosition(0));
        }
        return rc;
    }

    /**
     * Set an option in the store - key is saved in upper case (searches
     * are case insensitive).
     *
     * @param the name of the key to be saved, the name may contain only
     * a-z, A-Z, 0-9 and "." characters.
     * @param the value to be saved the name can contain no new-lines.
     * @return true if the value and parameter were acceptable format
     */

    public boolean setValue(String key, String value) {
        boolean rc;
        key = key.toUpperCase();
        // if (PafFile.valueKey(key) && PafFile.validValue(value) {
            store_.put(key, value);
            rc = true;
        // } else {
        //    rc = false;
        //   }
        return rc;
    }

    /**
     * Set a boolean value into the options.
     */

    public boolean setValue(String key, boolean val) {
        String valStr = "T";
        if (!val) {
            valStr = "F";
        }
        return setValue(key, valStr);
    }

    /**
     * Set an integer into the options
     */

    public boolean setValue(String key, int val) {
        return setValue(key, ""+val);
    }

    /**
     * Set a double into the options
     */

    public boolean setValue(String key, double val) {
        return setValue(key, ""+val);
    }

    /**
     * Set an ISO_DATE into the store.
     */

    public boolean setValue(String key, Date val) {
        SimpleDateFormat formatter
                = new SimpleDateFormat(Convert.ISO_DATE_FORMAT);
        return setValue(key, formatter.format(val));
    }
	
  /**
   * @return true if this string indicates the value useOBValue_
   */

  public static boolean isReadFromOB(String str) {
      return VisiPlotOptions.useOBValue.equalsIgnoreCase(str);
  }
  
  /**
   * @return true if val is an int between min and max
   * @param val the string containing the integer
   * @param min, minimum allowed value for val, inclusive.
   * @param max, maximum allowed value for val, inclusive.
   */
  
  public static boolean validInteger(String val, int min, int max) {
	boolean rc;
	try {
		int num = Integer.parseInt(val);
		rc = (num >= min && num <= max);
	} catch (NumberFormatException e) {
		rc = false;
	}
	return rc;
  }
  
  /**
   * @return true if val is a double between min and max
   * @param val the string containing the double
   * @param min, minimum allowed value for val, inclusive.
   * @param max, maximum allowed value for val, inclusive.
   */
  
  public static boolean validDouble(String val, double min, double max) {
	boolean rc;
	try {
		double num = Double.valueOf(val).doubleValue();
		rc = (num >= min && num <= max);
	} catch (NumberFormatException e) {
		rc = false;
	}
	return rc;
  }

  /**
   * Set the default values into the options class, clearing the store
   * before setting the new options.
   * Start and end times are set to today, 22h00 (UCT) start+12h respectively.
   */

  public void setDefaults() {
        store_.clear();

        Calendar today = Calendar.getInstance();
        // set to today.
        Date now = new Date();
        
        // subtract 12 hours so that "todays" date changes at 12h UTC, instead
        // of 0h UTC. This is a request (bug 2565) from Paranal and means that
        // "today" does not change halfway through the observing night.
        // Note the day will now change at 12 midday GMT - for Paranal this
        // means around 8am in the morning....which is acceptable.
        now.setTime(now.getTime() - TwelveHoursInMilliseconds);
        // debugging for the above problem.
        stdlog_.debug("Default plot date is the day of: "+now.toString());
        
        today.setTime(now);
        // set to 10pm UTC this evening
        today.set(Calendar.HOUR_OF_DAY, 22);
        today.set(Calendar.MINUTE, 0);
        today.set(Calendar.SECOND, 0);
        Date startTime = today.getTime();
        // set end time 12h later than the 22h start time.
        Date endTime = new Date(startTime.getTime()+(TwelveHoursInMilliseconds));
  
        setValue(    startTimeKey,          startTime           );
        setValue(    endTimeKey,            endTime             );
        setValue(    groupPlotKey,          true                );
        setValue(    sortPlotKey,           defaultSortOrder    );
        setValue(    airMassConKey,         useOBValue          );
        setValue(    airMassConEnabledKey,  true                );
        setValue(    moonDistConKey,        useOBValue          );
        setValue(    moonDistConEnabledKey, true                );
        setValue(    fliConKey,             useOBValue          );
        setValue(    fliConEnabledKey,      true                );
        setValue(    obexecConKey,          true                );
        setValue(    obtimeIntervalConKey,  false               );
        setValue(    obSiderealTimeIntervalConKey,  false       );        
        setValue(    sunConKey,             defaultSunCon       );
        setValue(    sunConEnabledKey,      true                );
        setValue(    dataQueryIntKey,       defaultPrecision    );
        setValue(    reportSeeingKey,       true                );
        setValue(    reportFLIKey,          false               );
        setValue(    reportMoonDistKey,     false               );
        setValue(    reportSkyTransKey,     true                );
        setValue(    reportAirMassKey,      true                );
        setValue(    reportRankKey,         true                );
        setValue(    reportProgIdKey,       true                );
        setValue(    reportOBIdKey,         true                );
        setValue(    reportRAKey,           true                );
        setValue(    reportDecKey,          true                );
        setValue(    reportExecTimeKey,     true                );
        setValue(    reportStartTimeKey,    true                );
        setValue(    printerName,           ""                  );
  }
}
