package org.eso.ohs.scripting;

/**
 * Adaptor class from TemplateAPI to TemplateSignature.
 * Class is initialized with a TemplateSignature and presents
 * the TemplateAPI interface
 *
 * @author Nick Kornweibel
 */

import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;

public class TemplateAdaptor implements TemplateAPI {

    private TemplateSignature tmpl_ = null;

    /**
     * Index of template in the observation block.
     * 0 for aquisition temaplate, > 0 for the Observation description.
     * -1 if not assigned or unknown.
     */
    private int idx_ = -1;

    /**
     * Constructor accepts a TemplateSignature object
     * @param temaplte signature, assume non-null
     */

    public TemplateAdaptor(TemplateSignature template, int id) {
        tmpl_ = template;
        setIdx(id);
    }

    public TemplateAdaptor(TemplateSignature template) {
        tmpl_ = template;
    }

    /**
     * Set the idx of this template in the OB.
     */

    public void setIdx(int id) {
        idx_ = id;
    }

    /**
     * @return index of template in the observation block.
     * 0 for aquisition temaplate, > 0 for the Observation description.
     * -1 if not assigned or unknown.
     */

    public int getIdx() {
        return idx_;
    }

    /**
     * Get the name of the template.
     * @return String containing the name of this template
     */

    public String getName() {
       return ScriptingUtilities.nullToEmptyString(tmpl_.getTemplateName());
    }

    /**
     * Get the id of this Template - simply returns the name again.
     * @return string containing the id of this template (long value as string)
     */

    public String getId() {
        //return ScriptingUtilities.nullToEmptyString(tmpl_.getTemplateName());
        return ""+getIdx();
    }


    /**
     * Get the value of a given parameter, or empty string if not found
     * @param the parameter name to locate
     * @return string containing the value for the parameter
     */

    public String getParameterValue(String key) {
        String rc = new String();
        Parameter[] params = tmpl_.getParamList();
        for (int i = 0; i < params.length; i++) {
            if (params[i].getParameterName().equals(key)) {
                rc = params[i].getValue();
                break;
            }
        }
        return rc;
    }
    
    
    
    /**
     * Implementation of TemplateAPI method
     * @see org.eso.ohs.scripting.TemplateAPI#setParameterValue(String, String)
     */
    
    
    public void setParameterValue(String key,String value) {   
           Parameter[] params = tmpl_.getDisplayParamList();
           for (int i = 0; i < params.length; i++) {
               if (params[i].getParameterName().equals(key)) {            
                   params[i].setValue(value);
                   break;
               }
           }           
    }

    /**
     * Get the number of parameters in this template
     * @return integer number of parameters
     */

    public int getNumberOfParameters() {
        return tmpl_.getParamList().length;
    }
 
    /**
     * Get parameter value for a specified parameter index, empty if
     * not a valid index
     * @param parameter index, starting from 0
     * @return string containing the value of the parameter
     */

    public String getParameterValueAt(int index) {
        String rc = new String();
        Parameter[] params = tmpl_.getParamList();
        if (index >= 0 && index < params.length) {
            rc = params[index].getValue();
        }
        return rc;
    }

    /**
     * Return a parameter value by index, or empty string if out of bounds.
     * @param index of parameter, starting from 0
     * @return string containing the name of the parameter
     */

    public String getParameterNameAt(int index) {
        String rc = new String();
        Parameter[] params = tmpl_.getParamList();
        if (index >= 0 && index < params.length) {
            rc = params[index].getParameterName();
        }
        return rc;
    }

    /**
     * Get the type of the Template - based on the TPL.TYPE keyword
     * in the template signature file.
     * @return the template type, or empty string if not known
     */

    public String getTemplateType() {
        return tmpl_.getTemplateType();
    }

    /**
     * @return the Acquisition time, in seconds, of this template, or
     * 0 if it is not known or template is not an acquisition template.
     */

    public int getAcquisitionTime() {
        return tmpl_.getAcquisitionTime();
    }
 
   
}
